@@ -14,35 +14,6 @@ fancy.cpt <- function( cpt )
   return( cpt )
 }
 
-
-# # Learn the CPTs of each node, given data, DAG, node sizes and equivalent sample size
-# # CPTs have the parents on dimensions 1:(n-1) and the child on the last dimension,
-# # so that the sum over the last dimension is always 1
-# learn.params <- function(data, dag, node.sizes, ess = 1)
-# {
-# 	# just to play safe
-# 	storage.mode(data) <- "integer"
-# 	storage.mode(dag) <- "integer"
-# 	storage.mode(node.sizes) <- "integer"
-# 	
-# 	n.nodes <- dim(data)[2]
-# 	cpts <- vector("list",n.nodes)
-#   var.names <- colnames(data)
-#   d.names <- mapply(function(name,size)(1:size),var.names,node.sizes)
-# 	# print(d.names)
-# 	# esimate a cpt for each family from data
-# 	for ( i in 1:n.nodes )
-# 	{
-# 		family <- c( which(dag[,i]!=0), i )
-# 		counts <- .Call( "compute_counts_nas", data[,family], node.sizes[family], 
-# 			PACKAGE = "bnstruct" )
-# 		cpts[[i]] <- counts.to.probs( counts + ess / prod(dim(counts)) )
-#     dimnames(cpts[[i]]) <- d.names[family]
-# 	}
-# 	names( cpts ) <- as.list(var.names)
-# 	return( cpts )
-# }
-
 # Compute Structural Hamming Distance between graphs g1 and g2
 #' compute the Structural Hamming Distance between two adjacency matrices.
 #' 
@@ -148,136 +119,6 @@ quantiles.matrix <- function(data, levels)
   return(quant)
 }
 
-# # Plot a weighted connectivity matrix using Rgraphviz
-# plot.mat <- function( mat, node.names = as.character(1:ncol(mat)), frac = 0.2, 
-#                       max.weight = max(mat), node.col = rep('white',ncol(mat)) )
-# {
-#   # check for Rgraphviz
-#   if (!require(Rgraphviz))
-#     stop("this function requires the Rgraphviz package.")
-#   
-#   # adjacency matrix
-#   mat.th <- mat
-#   mat.th[mat <  frac*max.weight] <- 0
-#   mat.th[mat >= frac*max.weight] <- 1
-#   # build graph
-#   g <- graphAM( mat.th, edgemode="directed")
-#   nodes(g) <- node.names
-#   en <- edgeNames(g,recipEdges="distinct")
-#   g <- layoutGraph(g)
-#   
-#   # set edge darkness proportional to confidence
-#   conf <- mat.th*pmax(mat,t(mat)) # both values to the maximum for edges with 2 directions
-#   col <- colors()[253-100*(t(conf)[t(conf) >= frac*max.weight]/max.weight)]
-#   names(col) <- en
-#   
-#   # remove arrowheads from undirected edges
-#   ahs <- edgeRenderInfo(g)$arrowhead
-#   ats <- edgeRenderInfo(g)$arrowtail
-#   dirs <- edgeRenderInfo(g)$direction
-#   ahs[dirs=="both"] <- ats[dirs=="both"] <- "none"
-#   edgeRenderInfo(g) <- list(col=col,lwd=2,arrowhead=ahs,arrowtail=ats)
-#   
-#   # node colors
-#   node.fill <- as.list(node.col)
-#   names(node.fill) <- node.names
-#   nodeRenderInfo(g) <- list(fill=node.fill)
-#   
-#   renderGraph(g)
-# }
-# 
-# dag.to.cpdag <- function(dag, layering = NULL)
-# {
-# 	return(abs(label.edges(dag, layering)))
-# }
-# 
-# label.edges <- function(dag, layering = NULL)
-# {
-# 	# LABEL-EDGES produce a N*N matrix which values are
-# 	# 	+1 if the edge is compelled or
-# 	#	-1 if the edge is reversible.
-# 
-# 	N<-nrow(dag)
-# 	o <- order.edges(dag)
-# 	order <- o$order
-# 	xedge <- o$x
-# 	yedge <- o$y
-# 	
-# 	label <- 2*dag
-# 	NbEdges <- length(xedge)
-# 	
-# 	# edges between layers are compelled
-# 	if( !is.null(layering) )
-# 	{
-# 		layers = length(unique(layering))
-# 		for( l in 1:(layers-1) )
-# 			label[ intersect(xedge,which(layering==l)), intersect(yedge,which(layering>l)) ] <- 
-# 				dag[ intersect(xedge,which(layering==l)), intersect(yedge,which(layering>l)) ]
-# 	} 
-# 	
-# 	for( Edge in 1:NbEdges)
-# 	{
-# 		xlow <- xedge[Edge]
-# 		ylow <- yedge[Edge]
-# 		if( label[xlow,ylow] == 2 )
-# 		{
-# 			fin <- 0
-# 			wcompelled <- which(label[,xlow] == 1)
-# 			parenty <- which(label[,ylow] != 0)
-# 			
-# 			for( s in seq_len(length(wcompelled)) )
-# 			{
-# 				w <- wcompelled[s]
-# 				if( !(w %in% parenty) )
-# 				{
-# 					label[parenty,ylow] <- 1
-# 					fin <- 1
-# 				}
-#             else if( fin == 0 ) label[w,ylow] <- 1
-# 			}
-# 			
-# 			if( fin == 0 )
-# 			{
-# 				parentx <- c(xlow,which(label[,xlow] != 0))
-# 				
-# 				if( length(setdiff(parenty,parentx) > 0) )
-#     				label[which(label[,ylow] == 2), ylow] <- 1
-# 				else
-# 				{
-# 					label[xlow,ylow] <- -1
-# 					label[ylow,xlow] <- -1
-# 					ttp <- which(label[,ylow] == 2)
-# 					label[ttp,ylow] <- -1
-# 					label[ylow,ttp] <- -1
-# 				}
-# 			}
-# 		}
-# 	}
-# 	return(label)
-# }
-# 
-# order.edges <- function(dag)
-# # ORDER_EDGES produce a total (natural) ordering over the edges in a DAG.
-# {
-# 	N <- nrow(dag)
-# 	order <- matrix(c(0),N,N)
-# 
-# 	node_order <- topological.sort(dag)
-# 	oo <- sort(node_order,index.return=TRUE)$ix
-# 	dag <- dag[oo,oo]
-# 	xy <- which(dag == 1, arr.ind = TRUE)
-# 	nb.edges <- nrow(xy)
-# 
-# 	if( nb.edges != 0)
-# 		order[xy] <- 1:nb.edges
-# 
-# 	order <- order[node_order,node_order]
-# 	x <- oo[xy[,1]]
-# 	y <- oo[xy[,2]]
-# 
-# 	return(list(order=order,x=x,y=y))
-# }
-
 ind2subv <- function(siz,index)
 {
 	# IND2SUBV   Subscript vector from linear index.
@@ -299,35 +140,6 @@ ind2subv <- function(siz,index)
 	return(sub)
 }
 
-# topological.sort <- function(dag)
-# # TOPOLOGICAL_SORT Return the nodes in topological order (parents before children).
-# {
-# 	n <- nrow(dag)
-# 	
-# 	# assign zero-indegree nodes to the top
-# 	fringe <- which( colSums(dag)==0 )
-# 	order <- rep(0,n)
-# 	
-# 	i <- 1
-# 	while( length(fringe) > 0 )
-# 	{
-# 		ind <- head(fringe,1) # pop
-# 		fringe <- tail(fringe,-1)
-# 		order[ind] <- i
-# 		i <- i + 1 
-# 		
-# 		for( j in which(dag[ind,] != 0) )
-# 		{
-# 			dag[ind,j] <- 0
-# 			if( sum(dag[,j]) == 0 )
-# 				fringe <- c(fringe,j)
-# 		}
-# 	}
-# 	
-# 	return(order)
-# }
-
-
 factors.to.graph <- function(factors, sep = '(')
 {
   # compute adjacency matrix from factor chain
@@ -465,3 +277,35 @@ fast.bincombinations <- function(p)
          FUN.VALUE = integer(2^(p))))
 }
 
+
+# identifies the connected components of a network structure,
+# in case the DAG is divided into disconnected subnetworks.
+# Return: a list whose elements are arrays containings the subgraphs
+identify.subgraphs <- function(am) {
+
+    visited <- rep(0, nrow(am))
+    subgs <- list()
+    csd <- c()
+    l <- 1
+
+    while (length(which(visited == 0)) > 0) {
+      curr <- which(visited == 0)[1]
+      visited[curr] <- l
+      csd = c(curr)
+      connected <- c(which(am[,curr] > 0), which(am[curr,] > 0))
+      while(length(connected) > 0) {
+        i <- connected[1]
+        connected <- connected[-1]
+        if (visited[i] == 0) {
+          connected <- unique(sort(c(connected,which(am[,i] > 0), which(am[i,] > 0))))
+          visited[i] <- l
+          csd <- c(csd, i)
+        }
+      } 
+      subgs[[l]] <- sort(csd)
+      csd <- c()
+      l <- l+1
+    }
+
+    return(subgs)
+}
