@@ -11,7 +11,7 @@
 SEXP bnstruct_score_node( SEXP data, SEXP node_sizes, SEXP ni, SEXP pars, SEXP func, SEXP ess );
 
 // compute score for a node as above, but called from c, not from R
-double score_node_1( int* data, int ncols_data, int nrows_data, int* node_sizes, unsigned int ni, int* pars, int length_pars, int func, double ess );
+double score_node_1( unsigned int* data, int ncols_data, int nrows_data, unsigned int* node_sizes, unsigned int ni, unsigned int* pars, int length_pars, int func, double ess );
 
 /* test wether the graph _g_ is acyclic
 */
